package system

import (
	"f5"
	"fmt"
	"main/constant"
	"main/model/system"
	"net/http"
	"q5"
	"strings"

	"github.com/gin-gonic/gin"
)

type GiftCodeApi struct {
	accountlocks q5.ConcurrentMap[string, int32]
	codelocks    q5.ConcurrentMap[string, int32]
}

func (gca *GiftCodeApi) Init() {
	gca.accountlocks = q5.ConcurrentMap[string, int32]{}
	gca.codelocks = q5.ConcurrentMap[string, int32]{}
}

func (gca *GiftCodeApi) GenCode(c *gin.Context) {
	var maxbatchid int32 = 0
	gameid := q5.SafeToInt32(c.DefaultQuery("gameid", "1"))
	typeid := q5.SafeToInt32(c.DefaultQuery("type", ""))
	gifttype := new(system.GiftType)
	db := f5.GetApp().GetOrmDb(constant.ADMIN_DB).Table("t_gift_type").Take(gifttype, "gameid = ? AND gift_type = ?", gameid, typeid)
	if db.Error != nil {
		if !f5.IsOrmErrRecordNotFound(db.Error) {
			f5.RspErr2(c, 1, db.Error.Error())
		} else {
			f5.RspErr2(c, 1, "type error")
		}

		return
	}

	db = f5.GetApp().GetOrmDb(constant.ADMIN_DB).Table("t_gift_code").Where("gameid = ?", gameid).Select("max(batch_id)")
	err := db.Error
	if err != nil {
		if !f5.IsOrmErrRecordNotFound(err) {
			f5.RspErr2(c, 1, err.Error())
			return
		}
	} else {
		var count int64 = 0
		if db.Count(&count); count > 0 {
			db.Scan(&maxbatchid)
		}
	}

	if maxbatchid >= 0xFFF {
		f5.RspErr2(c, 1, "batchid overflow")
		return
	}

	batchid := maxbatchid + 1

	count := q5.SafeToInt32(c.DefaultQuery("count", "1"))
	if count < 1 {
		count = 1
	}
	if count > 5000 {
		count = 5000
	}

	tryCount := 0
	codelist := map[string]int{}
	nowsecs := int32(f5.GetApp().GetRealSeconds())
	batchcodeinfo := []*system.GiftCode{}
	fixcode := fmt.Sprintf("%03x", batchid)
	sb := strings.Builder{}
	for i := 0; i < int(count); {
		if tryCount > 10000*10 {
			f5.RspErr2(c, 1, "internal error")
			return
		}

		sb.Reset()
		code := genCode(6)
		sb.WriteByte(fixcode[0])
		sb.WriteString(code[0:3])
		sb.WriteByte(fixcode[1])
		sb.WriteString(code[3:6])
		sb.WriteByte(fixcode[2])
		code = sb.String()
		_, exist := codelist[code]
		if exist {
			tryCount++
			continue
		}

		codelist[code] = 1
		i++
		p := new(system.GiftCode)
		p.GameId = gameid
		p.GiftCode = code
		p.Batch_id = batchid
		p.GiftType = typeid
		p.Limit = gifttype.Limit
		p.Count = 0
		p.Content = gifttype.Content
		p.CreateTime = nowsecs
		p.ModifyTime = nowsecs
		batchcodeinfo = append(batchcodeinfo, p)
	}

	err = f5.GetApp().GetOrmDb(constant.ADMIN_DB).Table("t_gift_code").Create(batchcodeinfo).Error
	if err != nil {
		f5.RspErr2(c, 1, err.Error())
		return
	}

	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "成功生成",
		"batchid": batchid,
	})
}

func (gca *GiftCodeApi) List(c *gin.Context) {
	req := struct {
		GameId  int32          `json:"gameid"`
		BatchId int32          `json:"batch_id"`
		PageDto system.PageDto `json:"page_dto"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		f5.RspErr2(c, 1, err.Error())
		return
	}
	sql := fmt.Sprintf("SELECT * FROM t_gift_code WHERE 1=1 AND gameid = %d AND batch_id = %d", req.GameId, req.BatchId)
	params := []string{}
	result := []*system.GiftCode{}
	f5.GetGoStyleDb().PageQuery(
		constant.ADMIN_DB,
		q5.ToInt32(req.PageDto.PageSize),
		q5.ToInt32(req.PageDto.Page),
		sql,
		params,
		f5.GetDbFilter().Comp([]f5.DbQueryFilter{}...),
		"",
		func(err error, pg *f5.Pagination) {
			if err != nil {
				f5.RspErr2(c, 1, err.Error())
				return
			}

			f5.UnmarshalModelList(pg.Rows, &result)

			c.JSON(http.StatusOK, gin.H{
				"code":       0,
				"message":    "获取成功",
				"data":       result,
				"total":      pg.Total,
				"total_page": pg.TotalPages,
				"cur_page":   pg.CurrentPage,
			})
		})
}

func (gca *GiftCodeApi) DownloadFile(c *gin.Context) {
	gameid := q5.SafeToInt32(c.DefaultQuery("gameid", ""))
	batchid := q5.SafeToInt32(c.DefaultQuery("batchid", ""))
	if gameid < 1 || batchid < 1 {
		f5.RspErr2(c, 1, "gameid or batchid error")
		return
	}

	sql := fmt.Sprintf("SELECT * FROM t_gift_code WHERE 1=1 AND gameid = %d AND batch_id = %d", gameid, batchid)
	f5.GetGoStyleDb().RawQuery(
		constant.ADMIN_DB,
		sql,
		[]string{},
		func(err error, ds *f5.DataSet) {
			if err != nil {
				f5.RspErr2(c, 1, err.Error())
				return
			}

			data := []*system.GiftCode{}
			f5.UnmarshalModelList(ds, &data)

			c.JSON(http.StatusOK, gin.H{
				"code":    0,
				"message": "",
				"data":    data,
			})
		})
}

func (gca *GiftCodeApi) ListType(c *gin.Context) {
	req := struct {
		GameId  int32          `binding:"required" json:"gameid"`
		PageDto system.PageDto `json:"page_dto"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		f5.RspErr2(c, 1, err.Error())
		return
	}
	sql := fmt.Sprintf("SELECT * FROM t_gift_type WHERE 1=1 AND gameid = %d", req.GameId)
	params := []string{}
	result := []*system.GiftType{}
	f5.GetGoStyleDb().PageQuery(
		constant.ADMIN_DB,
		q5.ToInt32(req.PageDto.PageSize),
		q5.ToInt32(req.PageDto.Page),
		sql,
		params,
		f5.GetDbFilter().Comp([]f5.DbQueryFilter{}...),
		"",
		func(err error, pg *f5.Pagination) {
			if err != nil {
				f5.RspErr2(c, 1, err.Error())
				return
			}

			f5.UnmarshalModelList(pg.Rows, &result)

			c.JSON(http.StatusOK, gin.H{
				"code":       0,
				"message":    "获取成功",
				"data":       result,
				"total":      pg.Total,
				"total_page": pg.TotalPages,
				"cur_page":   pg.CurrentPage,
			})
		})
}

func (gca *GiftCodeApi) AddType(c *gin.Context) {
	req := struct {
		GameId  int32  `binding:"required" json:"gameid"`
		Limit   *int32 `binding:"required" json:"limit"`
		Content string `binding:"required" json:"content"`
		Start   *int32 `binding:"required" json:"start"`
		End     *int32 `binding:"required" json:"end"`
		Comment string `binding:"required" json:"comment"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		f5.RspErr2(c, 1, err.Error())
		return
	}

	var maxtype int32 = 0
	db := f5.GetApp().GetOrmDb(constant.ADMIN_DB).Table("t_gift_type").Where("gameid = ?", req.GameId).Select("max(gift_type)")
	err := db.Error
	if err != nil {
		if !f5.IsOrmErrRecordNotFound(err) {
			f5.RspErr2(c, 1, err.Error())
			return
		}
	} else {
		var count int64 = 0
		if db.Count(&count); count > 0 {
			db.Scan(&maxtype)
		}
	}

	if maxtype >= 0xFFFF {
		f5.RspErr2(c, 1, "type overflow")
		return
	}

	nowsecs := int32(f5.GetApp().GetRealSeconds())

	p := new(system.GiftType)
	p.GiftType = maxtype + 1
	p.GameId = req.GameId
	p.Content = req.Content
	p.Limit = *req.Limit
	if p.Limit > 1 {
		p.Limit = 1
	}
	p.Start = *req.Start
	p.End = *req.End
	p.Comment = req.Comment
	p.CreateTime = nowsecs
	p.ModifyTime = nowsecs

	err = f5.GetApp().GetOrmDb(constant.ADMIN_DB).Table(p.TableName()).Create(p).Error
	if err != nil {
		f5.RspErr2(c, 1, err.Error())
		return
	}

	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "成功生成",
	})
}

func (gca *GiftCodeApi) QueryCode(c *gin.Context) {
	req := struct {
		GameId int32  `binding:"required" json:"gameid"`
		Code   string `binding:"required" json:"code"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		f5.RspErr(c, 1, err.Error())
		return
	}

	rspObj := struct {
		ErrCode    int32  `json:"errcode"`
		ErrMsg     string `json:"errmsg"`
		CodeStatus int32  `json:"status"` //0:可用;1:已用;2:不存在;3:失效
		Type       int32  `json:"type"`
	}{}
	codeinfo := new(system.GiftCode)
	db := f5.GetApp().GetOrmDb(constant.ADMIN_DB).Table(codeinfo.TableName()).Take(codeinfo, "gameid = ? AND gift_code = ?", req.GameId, req.Code)
	err := db.Error
	if err != nil {
		if !f5.IsOrmErrRecordNotFound(err) {
			f5.RspErr(c, 1, err.Error())
			return
		}
		rspObj.CodeStatus = 2
	} else if codeinfo.Limit > 0 && codeinfo.Count > 0 {
		rspObj.CodeStatus = 1
	} else {
		typeinfo := new(system.GiftType)
		db := f5.GetApp().GetOrmDb(constant.ADMIN_DB).Table(typeinfo.TableName()).Take(typeinfo, "gameid = ? AND gift_type = ?", req.GameId, codeinfo.GiftType)
		err := db.Error
		nowsec := int32(f5.GetApp().GetRealSeconds())
		if err != nil {
			if !f5.IsOrmErrRecordNotFound(err) {
				f5.RspErr(c, 1, err.Error())
				return
			}
			rspObj.CodeStatus = 2
		} else if (typeinfo.Start > 0 && typeinfo.Start > nowsec) ||
			(typeinfo.End > 0 && typeinfo.End < nowsec) {
			rspObj.CodeStatus = 3
		} else {
			rspObj.Type = codeinfo.GiftType
		}
	}

	c.JSON(http.StatusOK, rspObj)
}

func (gca *GiftCodeApi) UseCode(c *gin.Context) {
	req := struct {
		GameId    int32  `binding:"required" json:"gameid"`
		AccountId string `binding:"required" json:"account_id"`
		Code      string `binding:"required" json:"code"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		f5.RspErr(c, 1, err.Error())
		return
	}

	_, exist := gca.accountlocks.LoadOrStore(req.AccountId, 1)
	if exist {
		f5.RspErr(c, 2, "system busy")
		return
	}
	defer gca.accountlocks.Delete(req.AccountId)

	_, exist = gca.codelocks.LoadOrStore(req.Code, 1)
	if exist {
		f5.RspErr(c, 3, "system busy 2")
	}
	defer gca.codelocks.Delete(req.Code)

	rspObj := struct {
		ErrCode    int32  `json:"errcode"`
		ErrMsg     string `json:"errmsg"`
		CodeStatus int32  `json:"status"` //0:可用;1:已用;2:不存在;3:失效
		Limit      int32  `json:"limit"`
		Content    string `json:"content"`
		Type       int32  `json:"type"`
	}{}
	codeinfo := new(system.GiftCode)
	db := f5.GetApp().GetOrmDb(constant.ADMIN_DB).Table(codeinfo.TableName()).Take(codeinfo, "gameid = ? AND gift_code = ?", req.GameId, req.Code)
	err := db.Error
	if err != nil {
		if !f5.IsOrmErrRecordNotFound(err) {
			f5.RspErr(c, 1, err.Error())
			return
		}
		rspObj.CodeStatus = 2
	} else if codeinfo.Limit > 0 && codeinfo.Count >= codeinfo.Limit {
		rspObj.CodeStatus = 1
	} else {
		typeinfo := new(system.GiftType)
		db := f5.GetApp().GetOrmDb(constant.ADMIN_DB).Table(typeinfo.TableName()).Take(typeinfo, "gameid = ? AND gift_type = ?", req.GameId, codeinfo.GiftType)
		err := db.Error
		nowsec := int32(f5.GetApp().GetRealSeconds())
		if err != nil {
			if !f5.IsOrmErrRecordNotFound(err) {
				f5.RspErr(c, 1, err.Error())
				return
			}
			rspObj.CodeStatus = 2
		} else if (typeinfo.Start > 0 && typeinfo.Start > nowsec) ||
			(typeinfo.End > 0 && typeinfo.End < nowsec) {
			rspObj.CodeStatus = 3
		} else {
			rspObj.Limit = codeinfo.Limit
			rspObj.Content = codeinfo.Content
			rspObj.Type = codeinfo.GiftType
		}
	}

	c.JSON(http.StatusOK, rspObj)

	if rspObj.CodeStatus == 0 && codeinfo.Limit > 0 {
		codeinfo.Count++
		err := f5.GetApp().GetOrmDb(constant.ADMIN_DB).Table(codeinfo.TableName()).Where("gameid = ?", req.GameId).Where("gift_code = ?", req.Code).UpdateColumn("count", codeinfo.Count).Error
		if err != nil {
			f5.GetSysLog().Error("update gift code error:%s, %s", req.AccountId, req.Code)
		}
	}
}
