package activity

import (
	"f5"
	"main/common"
	"main/constant"
	"main/model"
	"main/mt"
	"main/service"
	"main/vo"
	"math/rand"
	"q5"

	"github.com/gin-gonic/gin"
)

type ActivityApi struct {
}

func (this *ActivityApi) RollDice(c *gin.Context) {
	s := c.MustGet(constant.SESSION_KEY).(common.Session)
	if s == nil {
		return
	}
	reqJson := struct {
		ForwardPoint int32 `json:"forward_point"`
	}{}
	if err := c.ShouldBindJSON(&reqJson); err != nil {
		f5.RspErr(c, 401, "params parse error")
		return
	}
	if reqJson.ForwardPoint < 0 || reqJson.ForwardPoint > 6 {
		f5.RspErr(c, 401, "forward_point param error")
		return
	}
	user := new(model.User)
	nowTime := f5.GetApp().GetRealSeconds()
	if err, found := user.Find(s.GetAccountId()); err != nil {
		f5.RspErr(c, 500, "server internal error1")
		return
	} else if !found {
		f5.RspErr(c, 500, "server internal error2")
		return
	}
	if reqJson.ForwardPoint == 0 && user.Dice <= 0 {
		f5.RspErr(c, 1, "dice not enough")
		return
	}
	
	score := mt.Table.Reward.RandScore() //rewardMeta.RandScore()
	if score <= 0 {
		f5.RspErr(c, 500, "server internal error4")
		return
	}
	if reqJson.ForwardPoint > 0 {
		if user.SpecDice <= 0 {
			f5.RspErr(c, 1, "spec dice not enough")
			return
		}
		if err := user.DecSpecDice(1); err != nil {
			f5.RspErr(c, 500, "server internal error5")
			return
		}
	} else if err := user.DecDice(1); err != nil {
		f5.RspErr(c, 500, "server internal error5")
		return
	}

	rspObj := struct {
		vo.BaseVo
		Point     int32 `json:"point"`
		AwardGrid int32 `json:"award_grid"`
	}{}
	rspObj.Point = int32(1 + rand.Intn(6))
	if reqJson.ForwardPoint > 0 {
		rspObj.Point = reqJson.ForwardPoint
	}

	var gridMeta *mt.MapGrid
	if user.CurrGrid+rspObj.Point > mt.Table.MapGrid.GetMaxGridId() {
		user.CurrGrid = user.CurrGrid + rspObj.Point - mt.Table.MapGrid.GetMaxGridId()
		gridMeta = mt.Table.MapGrid.GetById(int64(user.CurrGrid))
		if gridMeta == nil {
			f5.RspErr(c, 500, "server internal error7")
			return
		}
	} else {
		gridMeta = mt.Table.MapGrid.GetById(int64(user.CurrGrid + rspObj.Point))
		if gridMeta == nil {
			f5.RspErr(c, 500, "server internal error7")
			return
		}
		user.CurrGrid += rspObj.Point
	}
	
	user.AddFields("curr_grid")
	items := gridMeta.GetRewardItems()
	if items == nil {
		f5.RspErr(c, 500, "server internal error7")
		return
	}

	fullgrid := false
	scoreaward := true
	if len(*items) > 0 {
		gridawarded := false
		userAwardgrids := user.GetAwardGrids()
		for _, id := range userAwardgrids {
			if id == user.CurrGrid {
				gridawarded = true
				break
			}
		}

		if !gridawarded {
			for _, item := range *items {
				rspObj.GetOrCreateAward().AddItem(item.ItemId, item.ItemNum)
			}

			awardgridids := mt.Table.MapGrid.GetAwardGridIdList()
			user.AddAwardGrid(user.CurrGrid)
			userAwardgrids = user.GetAwardGrids()
			if len(awardgridids) <= len(userAwardgrids) {
				fullgrid = true
			}

			scoreaward = false
			rspObj.AwardGrid = 1
		}
	}

	{
		subtype := q5.SafeToString(constant.VIRTUAL_ITEM_DICE)
		cur := q5.SafeToString(user.Dice)
		if reqJson.ForwardPoint > 0 {
			subtype = q5.SafeToString(constant.VIRTUAL_ITEM_SPEC)
			cur = q5.SafeToString(user.SpecDice)
		}

		service.Log.AddAsyncLog(
			user.AccountId,
			constant.LOG_TYPE_ITEM,
			subtype,
			nowTime,
			[]string{
				constant.ITEM_OP_SOURCE_USE,
				"1",
				cur,
			})
	}

	if scoreaward {
		if err, ok := service.Buff.HasBuffEffect(s.GetAccountId(),
			constant.BUFF_EFFECT_SCORE_CARD_X3); err == nil {
			if ok {
				score *= 3
			}
		} else {
			f5.RspErr(c, 500, "server internal error7")
			return
		}
		rspObj.GetOrCreateAward().AddItem(constant.VIRTUAL_ITEM_SCORE, score)
	}

	op_src := constant.ITEM_OP_SOURCE_ROLL_DICE
	if reqJson.ForwardPoint > 0 {
		op_src = constant.ITEM_OP_SOURCE_ROLL_SPEC
	}
	bageffect, taskupdate, bufeffect := service.Award.HandleAwards(user, rspObj.GetOrCreateAward(), op_src, nil, nil)
	if bageffect {
		q5.AppendSlice(&rspObj.GetOrCreateSideEffect().Effects, constant.SIDE_EFFECT_BAG)
	}

	if bufeffect {
		q5.AppendSlice(&rspObj.GetOrCreateSideEffect().Effects, constant.SIDE_EFFECT_BUFF)
	}

	if fullgrid {
		q5.AppendSlice(&rspObj.BaseVo.GetOrCreateSideEffect().Effects, constant.SIDE_EFFECT_FULL_GRID)
	}

	itemtype := constant.ITEM_TYPE_SPEC_DICE
	if reqJson.ForwardPoint == 0 {
		itemtype = constant.ITEM_TYPE_DICE
	}

	if service.Task.UpdateTaskProgress(s.GetAccountId(), constant.TASK_COND_BACK_USE, int32(itemtype), 1, nil) || taskupdate {
		q5.AppendSlice(&rspObj.BaseVo.GetOrCreateSideEffect().Effects, constant.SIDE_EFFECT_TASK)
	}
	rspObj.GetOrCreateSideEffect().GetOrCreateUser().FromModel(user)

	c.JSON(200, rspObj)
}

func (this *ActivityApi) Claim(c *gin.Context) {
	s := c.MustGet(constant.SESSION_KEY).(common.Session)
	if s == nil {
		return
	}

	user := new(model.User)
	if err, found := user.Find(s.GetAccountId()); err != nil {
		f5.RspErr(c, 500, "server internal error1")
		return
	} else if !found {
		f5.RspErr(c, 500, "server internal error2")
		return
	}

	awardgridids := mt.Table.MapGrid.GetAwardGridIdList()
	userAwardgrids := user.GetAwardGrids()
	if len(awardgridids) != len(userAwardgrids) {
		f5.RspErr(c, 500, "bad reques")
		return
	}

	rspObj := vo.BaseVo{}

	gridMeta := mt.Table.MapGrid.GetById(0)
	if gridMeta == nil {
		f5.RspErr(c, 500, "server internal error7")
		return
	}

	items := gridMeta.GetRewardItems()
	if items == nil {
		f5.RspErr(c, 500, "server internal error7")
		return
	}

	user.CurrGrid = 1
	user.ClearAwardGrid()
	// user.UpdateFields([]string{"curr_grid", "award_grids"})
	user.AddFields("curr_grid")

	for _, item := range *items {
		rspObj.GetOrCreateAward().AddItem(item.ItemId, item.ItemNum)
	}

	bageffect, taskupdate, bufeffect := service.Award.HandleAwards(user, rspObj.GetOrCreateAward(), constant.ITEM_OP_SOURCE_FULL_GRID, nil, nil)
	if bageffect {
		q5.AppendSlice(&rspObj.GetOrCreateSideEffect().Effects, constant.SIDE_EFFECT_BAG)
	}

	if bufeffect {
		q5.AppendSlice(&rspObj.GetOrCreateSideEffect().Effects, constant.SIDE_EFFECT_BUFF)
	}

	present := user.CheckPresentDice()
	if present > 0 {
		award := new(vo.Award)
		award.AddItem(constant.VIRTUAL_ITEM_DICE, present)
		service.Award.HandleAwards(user, award, constant.ITEM_OP_SOURCE_PRESENT, nil, nil)
	}

	if taskupdate {
		q5.AppendSlice(&rspObj.GetOrCreateSideEffect().Effects, constant.SIDE_EFFECT_TASK)
	}

	rspObj.GetOrCreateSideEffect().GetOrCreateUser().FromModel(user)

	c.JSON(200, rspObj)
}
