package shop

import (
	"f5"
	"main/common"
	"main/constant"
	"main/model"
	"main/mt"
	"main/service"
	"main/vo"
	"q5"

	"github.com/gin-gonic/gin"
)

type ShopApi struct {
}

func (this *ShopApi) Goods(c *gin.Context) {
	s := c.MustGet(constant.SESSION_KEY).(common.Session)
	if s == nil {
		return
	}
	rspObj := struct {
		vo.BaseVo
		Data []*vo.Goods `json:"goods"`
	}{}
	mt.Table.Item.Traverse(func(itemMeta *mt.Item) bool {
		if itemMeta.GetItemType() != constant.ITEM_TYPE_CHIP {
			return true
		}
		goodsVo := new(vo.Goods)
		goodsVo.ItemId = itemMeta.GetId()
		q5.AppendSlice(&rspObj.Data, goodsVo)
		return true
	})
	c.JSON(200, rspObj)
}

func (this *ShopApi) Buy(c *gin.Context) {
	reqJson := struct {
		ItemId int32 `json:"item_id"`
	}{}
	if err := c.ShouldBindJSON(&reqJson); err != nil {
		f5.RspErr(c, 1, err.Error())
		return
	}
	s := c.MustGet(constant.SESSION_KEY).(common.Session)
	if s == nil {
		return
	}
	rspObj := struct {
		vo.BaseVo
	}{}
	user := new(model.User)
	
	if err, found := user.Find(s.GetAccountId()); err != nil {
		f5.RspErr(c, 500, "server internal error")
		return
	} else if !found {
		f5.RspErr(c, 500, "server internal error")
		return
	}
	itemMeta := mt.Table.Item.GetById(int64(reqJson.ItemId))
	if itemMeta == nil {
		f5.RspErr(c, 500, "item_id error1")
		return
	}
	if itemMeta.GetItemType() == constant.ITEM_TYPE_SCORE {
		f5.RspErr(c, 500, "item_id error2")
		return
	}

	if itemMeta.GetPriceType() != constant.TASK_AWARD_SCORE {
		f5.RspErr(c, 500, "item_id error3")
	}

	if itemMeta.GetPrice() < 0 {
		f5.RspErr(c, 500, "config error")
		return
	}
	if user.Score < int64(itemMeta.GetPrice()) {
		f5.RspErr(c, 1, "score not enough")
		return
	}
	if err := user.DecScore(itemMeta.GetPrice()); err != nil {
		f5.RspErr(c, 500, "server internal error5")
		return
	}

	award := new(vo.Award)
	award.AddItem(reqJson.ItemId, 1)
	bag, task, buff := service.Award.HandleAwards(user, award, constant.ITEM_OP_SOURCE_SHOP, nil, nil)
	if bag {
		q5.AppendSlice(&rspObj.BaseVo.GetOrCreateSideEffect().Effects, constant.SIDE_EFFECT_BAG)
	}

	if buff {
		q5.AppendSlice(&rspObj.BaseVo.GetOrCreateSideEffect().Effects, constant.SIDE_EFFECT_BUFF)
	}

	service.Log.AddAsyncLog(
		user.AccountId,
		constant.LOG_TYPE_ITEM,
		q5.SafeToString(constant.VIRTUAL_ITEM_SCORE),
		f5.GetApp().GetRealSeconds(),
		[]string{
			constant.ITEM_OP_SOURCE_SHOP,
			q5.SafeToString(itemMeta.GetPrice()),
			q5.SafeToString(user.Score),
			q5.SafeToString(itemMeta.GetId()),
		})

	rspObj.SideEffect = new(vo.SideEffect)
	rspObj.SideEffect.User = new(vo.User)
	rspObj.SideEffect.User.FromModel(user)
	rspObj.SideEffect.User.HourlyEarnings = service.Chip.GetHourEarning(user.AccountId)
	if service.Task.UpdateTaskProgress(s.GetAccountId(), constant.TASK_COND_BACK_BUY, itemMeta.GetItemType(), 1, nil) || task {
		q5.AppendSlice(&rspObj.BaseVo.GetOrCreateSideEffect().Effects, constant.SIDE_EFFECT_TASK)
	}

	c.JSON(200, rspObj)
}
