package user

import (
	"f5"
	"main/common"
	"main/constant"
	"main/model"
	"main/service"
	"main/vo"
	"q5"
	"wheelserver/mt"

	"github.com/gin-gonic/gin"
)

type UserApi struct {
}

func (this *UserApi) Login(c *gin.Context) {
	s := c.MustGet(constant.SESSION_KEY).(common.Session)
	if s == nil {
		return
	}
	user := new(model.User)
	rspObj := struct {
		vo.BaseVo
		UserInfo   vo.User `json:"user_info"`
		ServerTime int32   `json:"server_time"`
	}{}
	nowTime := f5.GetApp().GetRealSeconds()
	rspObj.ServerTime = int32(nowTime)
	err, found := user.Find(s.GetAccountId())
	if err != nil {
		f5.RspErr(c, 500, "server internal error")
		return
	} else if !found {
		user.AccountId = s.GetAccountId()
		user.Uid = s.GetUid()
		user.Gid = s.GetGid()
		user.OpenId = s.GetOpenId()
		user.Invited = s.GetInvited()
		user.Avatar = ""
		user.NickName = s.GetNickName()
		user.Score = 0
		user.Dice = constant.PRESENT_DICE_LIMIT //mt.Table.Global.GetDailyDiceNum()
		user.LastPresentDiceTime = q5.ToInt32(nowTime)
		user.CurrGrid = 1
		user.CreateTime = q5.ToInt32(nowTime)
		user.ModifyTime = q5.ToInt32(nowTime)
		if user.Create() != nil {
			f5.RspErr(c, 500, "server internal error")
			return
		}
		service.Log.AddAsyncLog(
			user.AccountId,
			constant.LOG_TYPE_ITEM,
			q5.SafeToString(constant.VIRTUAL_ITEM_DICE),
			nowTime,
			[]string{
				constant.ITEM_OP_SOURCE_BORN,
				q5.SafeToString(constant.PRESENT_DICE_LIMIT),
				q5.SafeToString(constant.PRESENT_DICE_LIMIT),
			})
		f5.GetMsgQueue().FireEvent(constant.MSG_CREATE_USER, q5.Args{user})
		service.Task.List(s.GetAccountId(), nil)
		if user.Invited != "" {
			service.Task.UpdateTaskProgress(user.Invited, constant.TASK_COND_BACK_GAIN, 0, 1, nil)
			invitor := new(model.User)
			err, found := invitor.Find(user.Invited)
			if err == nil && found {
				var meta *mt.Friend
				if s.GetPlatVip() != 0 {
					meta = mt.Table.Friend.GetMetaByType(constant.INVITEE_TYPE_VIP)
				}

				if meta == nil {
					meta = mt.Table.Friend.GetMetaByType(constant.INVITEE_TYPE_COMMON)
				}

				if meta != nil {
					if meta.GetRewardType() == constant.TASK_AWARD_SCORE {
						award := new(vo.Award)
						award.AddItem(constant.VIRTUAL_ITEM_SCORE, meta.GetReward())
						service.Award.HandleAwards(invitor, award, constant.ITEM_OP_SOURCE_INVITOR, nil, nil)
					}
				}
			}
			service.Friend.AddFriendLog(user.Invited, user.AccountId)
		}
	} else {
		if user.NickName != s.GetNickName() {
			user.UpdateName(s.GetNickName())
		}

		earning := service.Chip.CalcScore(s.GetAccountId())
		if earning > 0 {
			award := new(vo.Award)
			award.AddItem(constant.VIRTUAL_ITEM_SCORE, earning)
			_, task, _ := service.Award.HandleAwards(user, award, constant.ITEM_OP_SOURCE_CHIP_EARN, nil, nil)
			if task {
				q5.AppendSlice(&rspObj.BaseVo.GetOrCreateSideEffect().Effects, constant.SIDE_EFFECT_TASK)
			}
		}

		present := user.CheckPresentDice()
		if present > 0 {
			award := new(vo.Award)
			award.AddItem(constant.VIRTUAL_ITEM_DICE, present)
			service.Award.HandleAwards(user, award, constant.ITEM_OP_SOURCE_PRESENT, nil, nil)
		}
	}

	f5.GetMsgQueue().FireEvent(constant.MSG_LOGIN, q5.Args{user})
	rspObj.UserInfo.FromModel(user)
	if found {
		rspObj.UserInfo.HourlyEarnings = service.Chip.GetHourEarning(user.AccountId)
	}

	c.JSON(200, rspObj)
}

func (this *UserApi) Info(c *gin.Context) {
	s := c.MustGet(constant.SESSION_KEY).(common.Session)
	if s == nil {
		return
	}
	user := new(model.User)
	rspObj := struct {
		vo.BaseVo
		UserInfo vo.User `json:"user_info"`
	}{}

	if err, found := user.Find(s.GetAccountId()); err != nil {
		f5.RspErr(c, 500, "server internal error")
		return
	} else if !found {
		f5.RspErr(c, 1, "server internal error")
		return
	}

	present := user.CheckPresentDice()
	if present > 0 {
		award := new(vo.Award)
		award.AddItem(constant.VIRTUAL_ITEM_DICE, present)
		_, task, _ := service.Award.HandleAwards(user, award, constant.ITEM_OP_SOURCE_PRESENT, nil, nil)
		if task {
			q5.AppendSlice(&rspObj.GetOrCreateSideEffect().Effects, constant.SIDE_EFFECT_TASK)
		}
	}
	rspObj.UserInfo.FromModel(user)
	rspObj.UserInfo.HourlyEarnings = service.Chip.GetHourEarning(user.AccountId)
	c.JSON(200, rspObj)
}
