package middleware

import (
	"f5"
	"fmt"
	"jccommon"
	"main/mt"
	"main/common"
	"main/constant"
	"q5"

	. "main/global"

	"github.com/gin-gonic/gin"
)

func MaybeJwtAuth(c *gin.Context) {
	internalJwtAuth(c, true)
}

func JwtAuth(c *gin.Context) {
	internalJwtAuth(c, false)
}

/*
	'Authorization Bearer {JwtToken}'
*/
func internalJwtAuth(c *gin.Context, maybe bool) {
	tokenHeader := c.Request.Header.Get("Authorization")
	jwtToken := ""
	if len(tokenHeader) > 8 {
		jwtToken = tokenHeader[7:]
	} else {
		if maybe {
			s := GetSessionMgr().CreateGuestSession()
			c.Set(constant.SESSION_KEY, s)
			c.Next()
			return
		}
	}
	internalMetaMaskJwtAuth(c, jwtToken)
}

func internalMetaMaskJwtAuth(c *gin.Context, jwtToken string) {
	params := map[string]string{
		"c": "MetaMaskJwt",
		"a": "verify",
	}
	jsonReqObj := &struct {
		Data string `json:"data"`
	}{
		Data: jwtToken,
	}
	jsonRspObj := &struct {
		ErrCode interface{} `json:"errcode"`
		ErrMsg string `json:"errmsg"`
		Decoded *common.JwtUserInfo `json:"decoded"`
	}{}
	rspObj := &struct {
		ErrCode interface{} `json:"errcode"`
		ErrMsg string `json:"errmsg"`
	}{}
	paramsStr := q5.EncodeJson(jsonReqObj)
	url := fmt.Sprintf("%s/webapp/index.php", mt.Table.Web3ServiceCluster.RandElement().GetUrl())
	f5.GetHttpCliMgr().SendGoStyleJsonRspPost(
		url,
		params,
		jsonRspObj,
		q5.HTTP_HEADER_JSON,
		paramsStr,
		func(rsp f5.HttpCliResponse) {
			if rsp.GetErr() != nil ||
				!rsp.JsonParseOk() {
				rspObj.ErrCode = 500
				rspObj.ErrMsg = "server internal error"
				c.JSON(200, rspObj)
				c.Abort()
				return
			}
			rspObj.ErrCode = q5.SafeToInt32(jsonRspObj.ErrCode)
			rspObj.ErrMsg = jsonRspObj.ErrMsg
			if q5.SafeToInt32(rspObj.ErrCode) != 0 {
				rspObj.ErrCode = 501
				rspObj.ErrMsg = "jwt error"
				c.JSON(200, rspObj)
				c.Abort()
				return
			}
			if jsonRspObj.Decoded.Plat != jccommon.BC_POLY_POLY_TG {
				rspObj.ErrCode = 501
				rspObj.ErrMsg = "not support this platform"
				c.JSON(200, rspObj)
				c.Abort()
				f5.GetSysLog().Info("not support this platform:%s", rsp.GetRawData())
				return
			}
			s := GetSessionMgr().CreateJwtSession(jsonRspObj.Decoded)
			c.Set(constant.SESSION_KEY, s)
			c.Next()
		})
}
