package model

import (
	"errors"
	"f5"
	"main/constant"
	"q5"
	"strings"

	"gorm.io/gorm"
)

type TaskItem struct {
	MissionId int32 `json:"mission_id"`
	Current   int32 `json:"current"`
	Target    int32 `json:"target"`
	Award     int32 `json:"award"`
	Time      int64 `json:"time"`
}

type Task struct {
	Idx        int64  `gorm:"column:idx;AUTO_INCREMENT"`
	AccountId  string `gorm:"column:account_id;primaryKey"`
	Data       string `gorm:"column:data"`
	CreateTime int32  `gorm:"column:createtime;<-:create"`
	ModifyTime int32  `gorm:"column:modifytime"`
}

func (this *Task) TableName() string {
	return "t_task"
}

func (this *Task) UpdateFields(fields []string) error {
	if result := f5.GetApp().GetOrmDb(constant.WHEEL_DB).Model(this).Select(
		fields).Updates(this); result.Error != nil {
		return result.Error
	}
	return nil
}

func (this *Task) Find(accountId string) (error, bool) {
	if result := f5.GetApp().GetOrmDb(constant.WHEEL_DB).Table(this.TableName()).Take(
		this, "account_id = ?", accountId); result.Error != nil &&
		!errors.Is(result.Error, gorm.ErrRecordNotFound) {
		return result.Error, false
	} else {
		return nil, result.RowsAffected > 0
	}
}

func (this *Task) Create() error {
	if result := f5.GetApp().GetOrmDb(constant.WHEEL_DB).Create(this); result.Error != nil {
		return result.Error
	}
	return nil
}

func (t *Task) UpdateTasks(tasks map[int32]*TaskItem) error {
	if tasks == nil || len(tasks) < 1 {
		return errors.New("")
	}

	sb := strings.Builder{}
	for _, item := range tasks {
		sb.WriteString(q5.SafeToString(item.MissionId))
		sb.WriteByte(',')
		sb.WriteString(q5.SafeToString(item.Current))
		sb.WriteByte(',')
		sb.WriteString(q5.SafeToString(item.Target))
		sb.WriteByte(',')
		sb.WriteString(q5.SafeToString(item.Award))
		sb.WriteByte(',')		
		sb.WriteString(q5.SafeToString(item.Time))
		sb.WriteByte('|')
	}

	t.Data = sb.String()
	t.ModifyTime = int32(f5.GetApp().GetRealSeconds())
	if err := t.UpdateFields([]string{"data", "modifytime"}); err != nil {
		return err
	}

	return nil
}

func (this *Task) GetTaskItems() map[int32]*TaskItem {
	if this.Data != "" {
		Tasks := map[int32]*TaskItem{}
		taskitems := strings.Split(this.Data, "|")
		for _, itemstr := range taskitems {
			strs := strings.Split(itemstr, ",")
			if len(strs) < 5 {
				continue
			}
			taskitem := new(TaskItem)
			taskitem.MissionId = q5.SafeToInt32(strs[0])
			taskitem.Current = q5.SafeToInt32(strs[1])
			taskitem.Target = q5.SafeToInt32(strs[2])
			taskitem.Award = q5.SafeToInt32(strs[3])
			taskitem.Time = q5.SafeToInt64(strs[4])
			Tasks[taskitem.MissionId] = taskitem
		}

		return Tasks
	}

	return nil

}
