package model

import (
	"errors"
	"f5"
	"main/constant"
	"q5"
	"strings"

	"gorm.io/gorm"
)

type User struct {
	Idx                 int64          `gorm:"column:idx;AUTO_INCREMENT"`
	AccountId           string         `gorm:"column:account_id;primaryKey"`
	Uid                 string         `gorm:"column:uid"`
	Gid                 string         `gorm:"column:gid"`
	OpenId              string         `gorm:"column:openid"`
	Version             int32          `gorm:"column:version"`
	Avatar              string         `gorm:"column:avatar"`
	NickName            string         `gorm:"column:nickname"`
	Invited             string         `gorm:"column:invited"`
	Ext                 string         `gorm:"column:ext"`
	Plat                int32          `gorm:"column:plat"`
	PlatVip             int32          `gorm:"column:plat_vip"`
	Score               int64          `gorm:"column:score"`
	Dice                int32          `gorm:"column:dice"`
	SpecDice            int32          `gorm:"column:spec_dice"`
	LastPresentDiceTime int32          `gorm:"column:last_present_dice_time"`
	CurrGrid            int32          `gorm:"column:curr_grid"`
	AwardGrids          string         `gorm:"column:award_grids"`
	CreateTime          int32          `gorm:"column:createtime;<-:create"`
	ModifyTime          int32          `gorm:"column:modifytime"`
	updatefields        map[string]int `gorm:"-"`
}

func (this *User) TableName() string {
	return "t_user"
}

func (this *User) UpdateFields(fields []string) error {
	if result := f5.GetApp().GetOrmDb(constant.WHEEL_DB).Model(this).Select(
		fields).Updates(this); result.Error != nil {
		return result.Error
	}
	return nil
}

func (this *User) UpdateUserFields() error {
	if len(this.updatefields) == 0 {
		return nil
	}

	fields := []string{}
	for item := range this.updatefields {
		fields = append(fields, item)
	}

	for k := range this.updatefields {
		delete(this.updatefields, k)
	}	

	return this.UpdateFields(fields)
}

func (this *User) AddFields(f string) {
	if this.updatefields == nil {
		this.updatefields = map[string]int{}
	}
	this.updatefields[f] = 1
}

func (this *User) Find(accountId string) (error, bool) {
	if result := f5.GetApp().GetOrmDb(constant.WHEEL_DB).Table(this.TableName()).Take(
		this, "account_id = ?", accountId); result.Error != nil &&
		!errors.Is(result.Error, gorm.ErrRecordNotFound) {
		return result.Error, false
	} else {
		return nil, result.RowsAffected > 0
	}
}

func (this *User) CheckPresentDice() (present int32) {
	nowTime := int32(f5.GetApp().GetRealSeconds())
	if this.Dice < constant.PRESENT_DICE_LIMIT && this.LastPresentDiceTime+constant.PRESENT_DICE_LOOP < nowTime {
		present = (nowTime - this.LastPresentDiceTime) / constant.PRESENT_DICE_LOOP
		if this.Dice+present < constant.PRESENT_DICE_LIMIT {
			this.LastPresentDiceTime += present * constant.PRESENT_DICE_LOOP
		} else {
			present = constant.PRESENT_DICE_LIMIT - this.Dice
			this.LastPresentDiceTime = nowTime
		}

		this.AddFields("dice")
		this.AddFields("last_present_dice_time")
	}

	return present
}

func (this *User) Create() error {
	if result := f5.GetApp().GetOrmDb(constant.WHEEL_DB).Create(this); result.Error != nil {
		return result.Error
	}
	return nil
}

func (this *User) AddDice(num int32) error {
	if num < 1 {
		return errors.New("")
	}
	return this.DecDice(-num)
}

func (this *User) DecDice(num int32) error {
	if this.Dice < num {
		return errors.New("")
	}
	oldDice := this.Dice
	this.Dice -= num
	updatepresentTime := false
	if (num < 0 && oldDice < constant.PRESENT_DICE_LIMIT && this.Dice >= constant.PRESENT_DICE_LIMIT) ||
		(num > 0 && oldDice >= constant.PRESENT_DICE_LIMIT && this.Dice < constant.PRESENT_DICE_LIMIT) {
		this.LastPresentDiceTime = int32(f5.GetApp().GetRealSeconds())
		updatepresentTime = true
	}

	// updatefileds := []string{"dice"}
	this.AddFields("dice")
	if updatepresentTime {
		// updatefileds = append(updatefileds, "last_present_dice_time")
		this.AddFields("last_present_dice_time")
	}

	// if err := this.UpdateFields(updatefileds); err != nil {
	// 	this.Dice = oldDice
	// 	return err
	// }
	return nil
}

func (this *User) AddSpecDice(num int32) error {
	this.SpecDice += num
	// if err := this.UpdateFields([]string{"spec_dice"}); err != nil {
	// 	return err
	// }
	this.AddFields("spec_dice")
	return nil
}

func (this *User) DecSpecDice(num int32) error {
	if this.SpecDice < num {
		return errors.New("")
	}
	// oldSpecDice := this.SpecDice
	this.SpecDice -= num
	// if err := this.UpdateFields([]string{"spec_dice"}); err != nil {
	// 	this.SpecDice = oldSpecDice
	// 	return err
	// }
	this.AddFields("spec_dice")
	return nil
}

func (this *User) DecScore(num int32) error {
	if this.Score < int64(num) {
		return errors.New("")
	}
	// oldScore := this.Score
	this.Score -= int64(num)
	// if err := this.UpdateFields([]string{"score"}); err != nil {
	// 	this.Score = oldScore
	// 	return err
	// }
	this.AddFields("score")
	return nil
}

func (this *User) AddScore(score int32) error {
	if score <= 0 {
		return nil
	}
	// oldScore := this.Score
	this.Score += int64(score)
	// if err := this.UpdateFields([]string{"score"}); err != nil {
	// 	this.Score = oldScore
	// 	return err
	// }
	this.AddFields("score")
	return nil
}

func (this *User) UpdateName(name string) error {
	this.NickName = name
	// return this.UpdateFields([]string{"nickname"})
	this.AddFields("nickname")
	return nil
}

func (this *User) AddAwardGrid(gridid int32) error {
	awardGridsList := this.GetAwardGrids()
	if len(awardGridsList) > 0 {
		this.AwardGrids += ","
	}

	this.AwardGrids += q5.SafeToString(gridid)
	// return this.UpdateFields([]string{"award_grids"})
	this.AddFields("award_grids")
	return nil
}

func (this *User) ClearAwardGrid() error {
	this.AwardGrids = ""
	// return this.UpdateFields([]string{"award_grids"})
	this.AddFields("award_grids")
	return nil
}

func (this *User) GetAwardGrids() []int32 {
	gridstr := strings.Split(this.AwardGrids, ",")
	awardGridsList := []int32{}
	for _, item := range gridstr {
		if item != "" {
			awardGridsList = append(awardGridsList, q5.SafeToInt32(item))
		}
	}

	return awardGridsList
}
