package service

import (
	"f5"
	"main/constant"
	"main/model"
	"main/mt"
	"main/vo"
	"q5"
)

type award struct {
}

func (this *award) init() {
}

func (this *award) unInit() {
}

func (this *award) AddItem(accountId string, itemId int32, itemNum int32, baseVo *vo.BaseVo) {
	user := new(model.User)
	// nowTime := f5.GetApp().GetRealSeconds()
	if err, found := user.Find(accountId); err == nil && found {
		gmawards := new(vo.Award)
		gmawards.AddItem(itemId, itemNum)
		bag, task, buff := this.HandleAwards(user, gmawards, constant.ITEM_OP_SOURCE_GM, nil, nil)
		if baseVo != nil {
			if bag {
				baseVo.GetOrCreateAward().AddItem(itemId, itemNum)
				q5.AppendSlice(&baseVo.GetOrCreateSideEffect().Effects, constant.SIDE_EFFECT_BAG)
			} else if buff {
				q5.AppendSlice(&baseVo.GetOrCreateSideEffect().Effects, constant.SIDE_EFFECT_BUFF)
			} else {
				baseVo.GetOrCreateSideEffect().User = new(vo.User)
				baseVo.GetOrCreateSideEffect().User.FromModel(user)
				baseVo.GetOrCreateSideEffect().User.HourlyEarnings = Chip.GetHourEarning(user.AccountId)
			}

			if task {
				q5.AppendSlice(&baseVo.GetOrCreateSideEffect().Effects, constant.SIDE_EFFECT_TASK)
			}
		}
	}
}

func (aa *award) HandleAwards(user *model.User, totalaward *vo.Award, op_str string, appendlogparams []string, tasks *map[int32]*model.TaskItem) (bageffect bool, taskupdate bool, buffeffect bool) {
	nowTime := f5.GetApp().GetRealSeconds()
	for _, awarditem := range totalaward.Items {
		itemcfg := mt.Table.Item.GetById(int64(awarditem.ItemId))
		params := []string{
			op_str,
			q5.SafeToString(awarditem.ItemNum)}

		addlog := true
		switch itemcfg.GetItemType() {
		case constant.ITEM_TYPE_SCORE:
			user.AddScore(awarditem.ItemNum)
			params = append(params, q5.SafeToString(user.Score))

		case constant.ITEM_TYPE_DICE:
			user.AddDice(awarditem.ItemNum)
			params = append(params, q5.SafeToString(user.Dice))

		case constant.ITEM_TYPE_SPEC_DICE:
			user.AddSpecDice(awarditem.ItemNum)
			params = append(params, q5.SafeToString(user.SpecDice))

		case constant.ITEM_TYPE_CHIP:
			addlog = false
			for i := int32(0); i < awarditem.ItemNum; i++ {
				chip := new(model.Chip)
				chip.AccountId = user.AccountId
				chip.ItemId = awarditem.ItemId
				chip.ItemNum = 1
				chip.ExpireTime = int32(nowTime) + itemcfg.GetTime()*60
				chip.CreateTime = int32(nowTime)
				chip.ModifyTime = int32(nowTime)
				if chip.Create() != nil {
					f5.GetSysLog().Error("internal error, mapgrid chip award fail:%s, %d, %d", user.AccountId, user.CurrGrid, awarditem.ItemId)
				}

				logparams := []string{
					op_str,
					q5.SafeToString(chip.Idx),
				}

				if appendlogparams != nil && len(appendlogparams) > 0 {
					logparams = append(logparams, appendlogparams...)
				}
				Log.AddAsyncLog(
					user.AccountId,
					constant.LOG_TYPE_ITEM,
					q5.SafeToString(awarditem.ItemId),
					nowTime,
					logparams)
			}
		case constant.ITEM_TYPE_SCORE_DOUBLE_CARD:
			for i := int32(0); i < awarditem.ItemNum; i++ {
				Buff.Add(user.AccountId, itemcfg.GetBuffList())
			}
			buffeffect = true

			updated := Task.UpdateTaskProgress(user.AccountId, constant.TASK_COND_BACK_USE, constant.ITEM_TYPE_SCORE_DOUBLE_CARD, awarditem.ItemNum, nil)
			if !taskupdate {
				taskupdate = updated
			}

		case constant.ITEM_TYPE_SCORE_SMALL_GIFT_PACK:
			fallthrough
		case constant.ITEM_TYPE_SCORE_BIG_GIFT_PACK:
			bagItem := new(model.Bag)
			if err, found := bagItem.FindByItemId(user.AccountId, awarditem.ItemId); err == nil {
				if found {
					bagItem.AddItemNum(awarditem.ItemNum, int32(nowTime))
				} else {
					bagItem.AccountId = user.AccountId
					bagItem.ItemId = awarditem.ItemId
					bagItem.ItemNum = awarditem.ItemNum
					bagItem.CreateTime = int32(nowTime)
					bagItem.ModifyTime = int32(nowTime)
					bagItem.Create()
				}

				params = append(params, q5.SafeToString(bagItem.ItemNum)) // current
			}

			bageffect = true
		default:
			addlog = false
		}

		updated := Task.UpdateTaskProgress(user.AccountId, constant.TASK_COND_BACK_GAIN, itemcfg.GetItemType(), awarditem.ItemNum, tasks)
		if !taskupdate {
			taskupdate = updated
		}

		if addlog {
			if len(appendlogparams) > 0 {
				params = append(params, appendlogparams...)
			}

			Log.AddAsyncLog(
				user.AccountId,
				constant.LOG_TYPE_ITEM,
				q5.SafeToString(awarditem.ItemId),
				nowTime,
				params)
		}
	}

	user.UpdateUserFields()

	return bageffect, taskupdate, buffeffect
}
