package service

import (
	"f5"
	"main/constant"
	"main/model"
	"main/mt"
	"main/vo"
	"q5"
	"time"
)

type chip struct {
	earningUsers *q5.ConcurrentMap[string, int64] //[accountid, loginTime]
}

func (this *chip) init() {
	this.earningUsers = new(q5.ConcurrentMap[string, int64])
	go this.checkEarningLoop()
}

func (this *chip) unInit() {
}

func (this *chip) List(accountId string) (error, []*model.Chip) {
	nowTime := f5.GetApp().GetRealSeconds()
	chips := []*model.Chip{}
	result := f5.GetApp().GetOrmDb(constant.WHEEL_DB).Table(new(model.Chip).TableName()).Where(
		"account_id = ? AND expiretime > ?", accountId, nowTime).Find(&chips)
	return result.Error, chips
}

func (this *chip) CalcScore(accountId string) int32 {
	chips := []*model.Chip{}
	nowtime := int32(f5.GetApp().GetRealSeconds())
	result := f5.GetApp().GetOrmDb(constant.WHEEL_DB).Table(new(model.Chip).TableName()).Where(
		"account_id = ? AND expiretime > calc_time + createtime AND calc_time + createtime < ?", accountId, nowtime-600).Find(&chips)
	if result.Error == nil {
		var earningscore int32 = 0
		for _, chipitem := range chips {
			itemcfg := mt.Table.Item.GetById(int64(chipitem.ItemId))
			if itemcfg == nil {
				continue
			}

			var eclapse int32 = 0
			if nowtime < chipitem.ExpireTime {
				if nowtime > chipitem.CreateTime+chipitem.CalcTime {
					eclapse = nowtime - chipitem.CreateTime - chipitem.CalcTime
				}
			} else {
				if chipitem.CalcTime < itemcfg.GetTime() {
					eclapse = itemcfg.GetTime() - chipitem.CalcTime
				}
			}

			itemearning := (eclapse / 3600) * itemcfg.GetProduce() / (itemcfg.GetTime() / int32(60))
			if itemearning > 0 {
				earningscore += itemearning
				chipitem.CalcTime += eclapse / 3600 * 3600
				chipitem.ModifyTime = nowtime
				chipitem.UpdateCalcTime()
			}
		}

		return earningscore
	}

	return 0
}

func (this *chip) GetHourEarning(accountId string) string {
	err, chips := this.List(accountId)
	if err == nil {
		earnings := int32(0)
		for _, chipitem := range chips {
			itemcfg := mt.Table.Item.GetById(int64(chipitem.ItemId))
			earnings += itemcfg.GetProduce() / (itemcfg.GetTime() / int32(60))
		}

		if earnings > 0 {
			this.earningUsers.Store(accountId, f5.GetApp().GetRealSeconds())
		}
		return q5.SafeToString(earnings)
	}
	return "0"
}

func (this *chip) checkEarningLoop() {
	for {
		time.Sleep(time.Hour)
		nowtime := f5.GetApp().GetRealSeconds()
		deletelist := map[string]int32{}
		this.earningUsers.Range(func(accountId string, lastlogin int64) bool {
			if q5.GetDaySeconds(nowtime, 0) != q5.GetDaySeconds(lastlogin, 0) {
				deletelist[accountId] = 0
				return true
			}

			user := new(model.User)
			err, found := user.Find(accountId)
			if err != nil || !found {
				return true
			}
			
			userearning := this.CalcScore(accountId)
			if userearning > 0 {
				earnaward := new(vo.Award)
				earnaward.AddItem(constant.VIRTUAL_ITEM_SCORE, userearning)
				Award.HandleAwards(user, earnaward, constant.ITEM_OP_SOURCE_CHIP_EARN, nil, nil)
			}

			return true
		})

		for accoutid := range deletelist {
			this.earningUsers.Delete(accoutid)
		}
	}

}
