package service

import (
	"f5"
	"main/constant"
	"main/model"
	"main/mt"
	"main/vo"
	"q5"
	"strings"
)

type task struct {
}

func (this *task) init() {
}

func (this *task) unInit() {
}

func (this *task) List(accountId string, data *[]*vo.Mission) (errcode int32, msg string) {
	task := new(model.Task)
	err, found := task.Find(accountId)
	if err != nil {
		return 500, "server internal error"
	}

	createTasks := true
	nowtime := f5.GetApp().GetRealSeconds()
	tasksupdate := false
	newTask := func(tasks *map[int32]*model.TaskItem, taskMeta *mt.Task) {
		taskitem := new(model.TaskItem)
		taskitem.MissionId = taskMeta.GetId()
		taskitem.Current = 0
		taskitem.Target = taskMeta.GetParam1()
		taskitem.Award = 0
		taskitem.Time = f5.GetApp().GetRealSeconds()
		(*tasks)[taskMeta.GetId()] = taskitem
		tasksupdate = true
	}

	var tasks map[int32]*model.TaskItem
	if found {
		tasks = task.GetTaskItems()
		if tasks != nil {
			createTasks = false

			for taskid := range tasks {
				found := false
				mt.Table.Task.Traverse(func(taskMeta *mt.Task) bool {
					if taskid == taskMeta.GetId() {
						found = true

						if taskMeta.GetType() == constant.TASK_TYPE_DAILY &&
							q5.GetDaySeconds(tasks[taskid].Time, 0) != q5.GetDaySeconds(nowtime, 0) {
							tasks[taskid].Time = nowtime
							tasks[taskid].Current = 0
							tasks[taskid].Award = 0
							tasksupdate = true
						}
					}
					return true
				})

				if !found {
					delete(tasks, taskid)
					tasksupdate = true
				}
			}

			mt.Table.Task.Traverse(func(taskMeta *mt.Task) bool {
				taskid := taskMeta.GetId()

				_, ok := tasks[taskid]
				if !ok {
					newTask(&tasks, taskMeta)
				}

				return true
			})
		}
	}

	if createTasks {
		tasks = map[int32]*model.TaskItem{}
		mt.Table.Task.Traverse(func(taskMeta *mt.Task) bool {
			newTask(&tasks, taskMeta)
			return true
		})
		task.AccountId = accountId
		task.CreateTime = int32(nowtime)
		task.ModifyTime = int32(nowtime)
		sb := strings.Builder{}
		for _, item := range tasks {
			sb.WriteString(q5.SafeToString(item.MissionId))
			sb.WriteByte(',')
			sb.WriteString(q5.SafeToString(item.Current))
			sb.WriteByte(',')
			sb.WriteString(q5.SafeToString(item.Target))
			sb.WriteByte(',')
			sb.WriteString(q5.SafeToString(item.Award))
			sb.WriteByte(',')
			sb.WriteString(q5.SafeToString(item.Time))
			sb.WriteByte('|')
		}

		task.Data = sb.String()
		task.Create()
		tasksupdate = false
	}

	if data != nil {
		for _, taskitem := range tasks {
			if taskitem.Award != 0 {
				taskcfg := mt.Table.Task.GetById(int64(taskitem.MissionId))
				if taskcfg.GetType() == constant.TASK_TYPE_ONCE {
					// 已领奖的一次性任务不再显示
					continue
				}
			}
			missionVo := new(vo.Mission)
			missionVo.FromModel(taskitem)
			q5.AppendSlice(data, missionVo)			
		}

		q5.Sort(*data, func(t *vo.Mission, m *vo.Mission) bool {
			return t.MissionId < m.MissionId
		})
	}

	if tasksupdate {
		task.UpdateTasks(tasks)
	}

	return 0, ""
}

func (this *task) UpdateTaskProgress(accountId string, condition int32, itemtypeid int32, amount int32, extasks *map[int32]*model.TaskItem) (taskupdate bool) {
	var tasks map[int32]*model.TaskItem
	var task *model.Task
	if extasks == nil {
		task = new(model.Task)
		err, found := task.Find(accountId)
		if err != nil || !found {
			return false
		}

		tasks = task.GetTaskItems()
		if tasks == nil {
			return false
		}
	}

	for _, taskitem := range tasks {
		if taskitem.Award != 0 || taskitem.Current >= taskitem.Target {
			continue
		}

		taskMeta := mt.Table.Task.GetById(int64(taskitem.MissionId))
		if taskMeta == nil {
			continue
		}

		if taskMeta.GetCondition() != condition {
			continue
		}

		if taskMeta.GetParam2() != itemtypeid {
			continue
		}

		taskitem.Current += amount
		if taskitem.Current > taskitem.Target {
			taskitem.Current = taskitem.Target
		}

		taskupdate = true
	}

	if taskupdate && extasks == nil {
		task.UpdateTasks(tasks)
	}

	return taskupdate
}
